/*
  François Pessaux 03/2005.
  This code can be freely distributed.

  Turn on/off a LED by interruption triggered by pressign on a switch branched
  on digital pin 2.
  Uses the hardware debounce with the Arduino Due.

  ATTENTION: #define DUE if running on an Arduno Due otherwise do not #define
    anything special.

  ATTENTION: #define MONITOR_ITS_NUM to write the number of detected interrupts
    on the serial. Otherwise, only the LED blink when the button is
    pressed/released.
*/

#define DUE
#define MONITOR_ITS_NUM

/* Plug the LED on pin 13. */
#define LED_PIN (13)
/* On Uno, use pin 2 which is attached to interrupt 0. On Due, we will
   also use pin 2 but will directly set it when calling attachInterrupt. */
#define BUTTON_PIN (2)


#ifdef MONITOR_ITS_NUM
volatile unsigned short counter = 0 ;
#endif


/* Interrupt Service Routine (ISR). */
void pinChange ()
{
#ifdef MONITOR_ITS_NUM
  counter++ ;
#endif
  /* Change the pin state. */
  if (digitalRead (BUTTON_PIN) == HIGH) digitalWrite (LED_PIN, HIGH) ;
  else digitalWrite (LED_PIN, LOW) ;
}

void setup ()
{
  /* First, turn the LED on in case the line was initially high. */
  pinMode (LED_PIN, OUTPUT) ;
  digitalWrite (LED_PIN, LOW) ;      /* Configure the ED pin as an output. */

#ifdef DUE
  /* Pin 2 is bit 25 of PIO B. */
  /* See p 647, 648. */
  REG_PIOB_IFER |= 0x1 << 25;   /* Input Filter Enable Register. */
  REG_PIOB_DIFSR = 0x1 << 25 ;  /* Debouncing Input Filter Select Register. */
  REG_PIOB_SCDR = 0xFF ;        /* Slow Clock Divider Register. */
#endif

  /* Attach the interrupt handler. */
#ifdef DUE
  /* On the Due, instead of specifying the IT number, we directly specify the
     pin onto attach the interrupt. */
  attachInterrupt (BUTTON_PIN, pinChange, CHANGE) ;
#else
  /* On Uno, we specify the IT number. */
  attachInterrupt (0, pinChange, CHANGE) ;
#endif
#ifdef MONITOR_ITS_NUM
  Serial.begin (9600) ;
#endif
}


void loop ()
{
  /* Nothing to do. */
#ifdef MONITOR_ITS_NUM
  Serial.println (counter) ;
#endif
}
